import {h, build, renderToString, Text} from 'ink'
import test from 'ava'
import {spy} from 'sinon'

import InkSelect from '.'

// Tests ---------------------------------------------------------------------

test('Placeholder.', t => {
  t.is(renderToString(<InkSelect placeholder={'Type a country'}/>), renderToString((
    <span>
      <Text/>
      <Text dim>{'Type a country'}</Text>
    </span>
	)))
})

test('Value.', t => {
  t.is(renderToString(<InkSelect value={'Sl'} items={[]}/>), renderToString((
    <span>
      <Text>{'Sl'}</Text>
      <Text dim/>
    </span>
	)))
})

test('Matching basic suggestion.', t => {
  const items = ['Slovenia', 'United Kingdom', 'Germany']

  t.is(renderToString(<InkSelect value={'Sl'} items={items}/>), renderToString((
    <span>
      <Text green>{'Sl'}</Text>
      <Text dim>{'ovenia'}</Text>
    </span>
	)))
})

test('Matching object suggestion.', t => {
  const items = [
    {country: 'Slovenia', capital: 'Ljubljana'},
    {country: 'United Kingdom', capital: 'London'},
    {country: 'Germany', capital: 'Berlin'}
  ]

  const getItemValue = item => item.country

  t.is(renderToString(<InkSelect value={'Sl'} items={items} getItemValue={getItemValue}/>), renderToString((
    <span>
      <Text green>{'Sl'}</Text>
      <Text dim>{'ovenia'}</Text>
    </span>
	)))
})

test('Not matching suggestion.', t => {
  const items = ['Slovenia', 'United Kingdom', 'Germany']

  t.is(renderToString(<InkSelect value={'Cr'} items={items}/>), renderToString((
    <span>
      <Text>{'Cr'}</Text>
      <Text dim/>
    </span>
	)))
})

test('On change trigger.', t => {
  const setRef = spy()

  build(<InkSelect ref={setRef}/>)

  const ref = setRef.firstCall.args[0]
  const propsRef = setRef.firstCall.args[0].props

  spy(propsRef, 'onChange')

  ref.handleKeyPress('S', {
    name: 'S',
    sequence: 'S'
  })

  t.true(propsRef.onChange.calledOnce)
  t.deepEqual(propsRef.onChange.firstCall.args[0], 'S')
})

test('On suggestion trigger (tab).', t => {
  const items = ['Slovenia', 'United Kingdom', 'Germany']
  const setRef = spy()

  build(<InkSelect ref={setRef} value={'U'} items={items}/>)

  const ref = setRef.firstCall.args[0]
  const propsRef = setRef.firstCall.args[0].props

  spy(propsRef, 'onChange')

  ref.handleKeyPress('', {
    name: 'tab'
  })

  t.true(propsRef.onChange.calledOnce)
  t.deepEqual(propsRef.onChange.firstCall.args[0], 'United Kingdom')
})

test('On suggestion trigger (right).', t => {
  const items = ['Slovenia', 'United Kingdom', 'Germany']
  const setRef = spy()

  build(<InkSelect ref={setRef} value={'U'} items={items}/>)

  const ref = setRef.firstCall.args[0]
  const propsRef = setRef.firstCall.args[0].props

  spy(propsRef, 'onChange')

  ref.handleKeyPress('', {
    name: 'right'
  })

  t.true(propsRef.onChange.calledOnce)
  t.deepEqual(propsRef.onChange.firstCall.args[0], 'United Kingdom')
})

test('On submit trigger with matching suggestion.', t => {
  const items = [
    {country: 'Slovenia', capital: 'Ljubljana'},
    {country: 'United Kingdom', capital: 'London'},
    {country: 'Germany', capital: 'Berlin'}
  ]

  const getItemValue = item => item.country
  const setRef = spy()

  build(<InkSelect ref={setRef} value={'United Kingdom'} items={items} getItemValue={getItemValue}/>)

  const ref = setRef.firstCall.args[0]
  const propsRef = setRef.firstCall.args[0].props

  spy(propsRef, 'onSubmit')

  ref.handleKeyPress('', {
    name: 'return'
  })

  t.true(propsRef.onSubmit.calledOnce)
  t.deepEqual(propsRef.onSubmit.firstCall.args[0], items[1])
})

test('On submit trigger not called with no matching suggestion.', t => {
  const items = ['Slovenia', 'United Kingdom', 'Germany']
  const setRef = spy()

  build(<InkSelect ref={setRef} value={'Cr'} items={items}/>)

  const ref = setRef.firstCall.args[0]
  const propsRef = setRef.firstCall.args[0].props

  spy(propsRef, 'onSubmit')

  ref.handleKeyPress('', {
    name: 'return'
  })

  t.true(propsRef.onSubmit.notCalled)
})

// ---------------------------------------------------------------------------
